    'SPEAKER SWITCH v1.0
    'A script to control the audio output of a
    'switch box connected to the parallel port
    '
    'USAGE : SpeakerSwitch.vb <DeviceCode> <on/off>
    'This will switch the speaker ports on the 
    'SpeakerBox ON/OFF.
    '
    'SEE ZMC.INI FOR SETTINGS
    Public Sub Main(ByVal Parms As Object)
        Const Debug = 0  ' 0 for OFF and 1 for ON
        Const LPT1 = 378 ' Parallel poort address
        Const AAN = 2 ' ON is a reverse keyword so i used the Dutch word for it
        Const OFF = 3

        '--------------------------
        'Get commandline arguments
        '--------------------------
        Dim ParmArray() As String
        Dim HouseCode As String
        Dim Status As String
        Dim State = 1 'ON selected
        ParmArray = Parms.ToString.Split(" ")
        HouseCode = UCase(ParmArray(0))
        Status = UCase(ParmArray(1))
        If (Status = "OFF") Then State = 0
        'DEBUGGING
        If (Debug = 1) Then
            hs.WriteLog("SpeakerBox DEBUG", "SpeakerBox - House Code:" & HouseCode & " Status : " & Status)
        End If
        'END DEBUGGING

        '--------------------------
        'Get the room settings
        '--------------------------
        Dim NumberOfRooms = 0
        While UCase(hs.GetINISetting("Settings", "Room" & NumberOfRooms, "", "ZMC.ini")) <> ""
            NumberOfRooms = NumberOfRooms + 1
        End While

        'We have one room less
        NumberOfRooms = NumberOfRooms - 1
        Dim Count = 0
        Dim Code(NumberOfRooms) As String
        Dim X10Out(NumberOfRooms) As String
        Dim Manual(NumberOfRooms) As Integer

        For Count = 0 To NumberOfRooms
            Code(Count) = UCase(hs.GetINISetting("Settings", "Room" & Count, "", "ZMC.ini"))
            X10Out(Count) = hs.GetINISetting("Settings", "X10Out" & Count, "", "ZMC.ini")
            Manual(Count) = Val(hs.GetINISetting("Settings", "Manual" & Count, "", "ZMC.ini"))
        Next Count
        'DEBUGGING
        If (Debug = 1) Then
            For Count = 0 To NumberOfRooms
                hs.WriteLog("SpeakerBox DEBUG", "SpeakerBox ZMC.ini- Room" & Count & "=" & Code(Count))
                hs.WriteLog("SpeakerBox DEBUG", "SpeakerBox ZMC.ini- X10Out" & Count & "=" & X10Out(Count))
                hs.WriteLog("SpeakerBox DEBUG", "SpeakerBox ZMC.ini- Manual" & Count & "=" & Manual(Count))
            Next Count
        End If
        'END DEBUGGING

        '--------------------------
        'Is the radio already ON?
        '--------------------------
        CheckIfRadioIsON()

        '--------------------------
        'What is the present status
        'of the Speakerbox output?
        '--------------------------
        Dim SpeakerStatus As Integer
        SpeakerStatus = Val(hs.GetINISetting("Status", "SpeakerStatus", "", "ZMC.ini"))
        'DEBUGGING
        If (Debug = 1) Then
            hs.WriteLog("SpeakerBox DEBUG", "SpeakerBox ZMC.ini- SpeakerStatus was " & SpeakerStatus)
        End If
        'END DEBUGGING

        '--------------------------
        'Find the right bit
        '--------------------------
        Dim BitFound As Boolean
        Dim Channel = -1
        BitFound = True
        Count = 0
        While (Code(Count) <> HouseCode)
            Count = Count + 1
        End While

        'Check for the AudioBus
        Dim AudioBus As Integer
        Channel = Count
        If (Count > 5) Then
            Channel = 6 'Audiobus with address code 64
            AudioBus = Val(hs.GetINISetting("Status", "AudioBus", "", "ZMC.ini"))
            hs.WriteLog("SpeakerBox", "SpeakerBox - AudioBus read is " & AudioBus.ToString)
            SetStatus(AudioBus, Count - 5, State)
        End If

        'If there isn't a known room found
        If (Count = -1) Then
            BitFound = False
            hs.WriteLog("SpeakerBox", "Could not find House Code:" & HouseCode & " - Error in ZMC.ini")
        End If


        '--------------------------
        'A known room is found
        '--------------------------
        If (BitFound = True) Then
            hs.WriteLog("SpeakerBox", "SpeakerBox - Status read is " & SpeakerStatus.ToString)

            'Turn ON the right bit
            SetStatus(SpeakerStatus, Channel + 1, State)
            'Is this channel NOT MUTED
            If (Manual(Count) = 0) Then
                Dim SpeakerBox As String
                'Get the application filename and path to use
                SpeakerBox = hs.GetINISetting("Settings", "SpeakerBox", "", "ZMC.ini")
                hs.Launch(SpeakerBox, "write " & LPT1 & " " & SpeakerStatus.ToString)
                hs.SaveINISetting("Status", "SpeakerStatus", SpeakerStatus, "ZMC.ini")
                hs.WriteLog("SpeakerBox", "SpeakerBox - Status send is " & SpeakerStatus.ToString)
                hs.SaveINISetting("Status", "AudioBus", AudioBus, "ZMC.ini")
                hs.WriteLog("SpeakerBox", "SpeakerBox - AudioBus write is " & AudioBus.ToString)
                'Are there additional actions to take?
                If (X10Out(Count) <> "") Then
                    If (State = 1) Then
                        hs.SetDeviceStatus(X10Out(Count), AAN)
                    Else
                        hs.SetDeviceStatus(X10Out(Count), OFF)
                    End If
                End If
            End If
        End If

        '--------------------------
        'Turn WMP OFF if there is
        'no room anymore where
        'music is playing.
        '--------------------------
        If (SpeakerStatus = 0) And (AudioBus = 0) Then hs.TriggerEvent("Muziek speler OFF")
    End Sub

    Sub CheckIfRadioIsON()
        'Check if radio is ON
        'If not then turn it on with provided URL
        'and update ZMC.ini file

        Dim WMPAan
        Dim LastStation
        Dim UsePlayer

        WMPAan = hs.GetINISetting("Status", "WMPAan", "", "ZMC.ini")

        If WMPAan = 0 Then
            LastStation = hs.GetINISetting("Status", "LastStation", "", "ZMC.ini")
            UsePlayer = hs.GetINISetting("Settings", "UsePlayer", "", "ZMC.ini")
            hs.Launch(UsePlayer, LastStation)
            hs.SaveINISetting("Status", "WMPAan", 1, "ZMC.ini")
            'Set no state to NO ROOM have yet music
            hs.SaveINISetting("Status", "SpeakerStatus", "0", "ZMC.ini")
            hs.WriteLog("SpeakerBox", "Music player (" & UsePlayer & ") turned ON and playing station:" & LastStation)
        End If
    End Sub

    Sub SetStatus(ByRef SpeakerStatus, ByVal Bit, ByVal state)
        If (state = 1) Then
            SetBit(SpeakerStatus, Bit)
        Else
            ClearBit(SpeakerStatus, Bit)
        End If
    End Sub

    Sub ClearBit(ByRef MyByte, ByVal MyBit)
        Dim BitMask As Int16
        ' Create a bitmask with the 2 to the nth power bit set:
        BitMask = 2 ^ (MyBit - 1)
        ' Clear the nth Bit:
        MyByte = MyByte And Not BitMask
    End Sub

    ' The SetBit Sub will set the 1 based, nth bit 
    ' (MyBit) of an integer (MyByte).
    Sub SetBit(ByRef MyByte, ByVal MyBit)
        Dim BitMask As Int16
        BitMask = 2 ^ (MyBit - 1)
        MyByte = MyByte Or BitMask
    End Sub

    